import sys
import numpy as np
from numpy import sqrt, exp, sin, cos, tan, pi
import numpy.linalg as LA 
import csv
from matplotlib import pyplot as plt


#===================================
# physical constants
#===================================
pi   = 3.14159265358979323846
pi2  = 2.0 * pi
h    = 6.6260755e-34    # Js";
hbar = 1.05459e-34      # "Js";
c    = 2.99792458e8     # m/s";
e    = 1.60218e-19      # C";
e0   = 8.854418782e-12; # C<sup>2</sup>N<sup>-1</sup>m<sup>-2</sup>";
kB   = 1.380658e-23     # JK<sup>-1</sup>";
me   = 9.1093897e-31    # kg";
R    = 8.314462618      # J/K/mol
a0   = 5.29177e-11      # m";


#===================================
# parameters
#===================================
a   = 4.0 # A
h12 = 2.0 # eV

kmin = -0.5
kmax =  0.5
nk   = 101

outfile = 'band.csv'


#===================================
# figure configuration
#===================================
fontsize        = 12
legend_fontsize = 8


#=============================
# other functions
#=============================
def savecsv(outfile, header, datalist):
    try: 
        print("Write to [{}]".format(outfile))
        f = open(outfile, 'w')
    except:
#    except IOError:
        print("Error: Can not write to [{}]".format(outfile))
    else:
        fout = csv.writer(f, lineterminator='\n')
        fout.writerow(header)
#        fout.writerows(data)
        for i in range(0, len(datalist[0])):
            a = []
            for j in range(len(datalist)):
                a.append(datalist[j][i])
            fout.writerow(a)
        f.close()


def main():
    global a, h12
    global kmin, kmax, nk

    kstep = (kmax - kmin) / (nk - 1)
    
    xk     = []
    yELCAO = []
    print("")
    print("LCAO band")
    print("k   E (eV)")
    for i in range(nk):
        k = kmin + i * kstep
        E = 2.0 * h12 * (1.0 - cos(2.0 * pi * k))
        xk.append(k)
        yELCAO.append(E)
        print("{:6.3f}  {:8.4f}".format(k, E))

    meff = hbar * hbar / 2.0 / (h12 * e) / (a * 1.0e-10)**2 / me
    yEFree = []
    print("")
    print("Free electron band")
    print("m* = {} me".format(meff))
    print("k   E (eV)")
    kK = 2.0 * pi / (a * 1.0e-10) # m^-1
    for i in range(nk):
        k = kmin + i * kstep
        P = hbar * kK * k
        E = P * P / 2.0 / (meff * me) / e
        yEFree.append(E)
        print("{:6.3f}  {:8.4f}".format(k, E))

    print("")
    print("Save to [{}]".format(outfile))
    savecsv(outfile, ["k", "ELCAO (eV)", "EFree (eV)"], [xk, yELCAO, yEFree])
    print("")
    print("plot")
    fig = plt.figure(figsize = (8, 4))
    ax1 = fig.add_subplot(1, 1, 1)

    ax1.plot(xk, yELCAO, label = 'LCAO')
    ax1.plot(xk, yEFree, label = 'Free')
    ax1.set_ylim([0.0, max(yELCAO) * 1.2])
    ax1.set_xlabel("k", fontsize = fontsize)
    ax1.set_ylabel("E (eV)", fontsize = fontsize)
#    plt.tight_layout()

    plt.pause(0.1)
    input()



if __name__ == "__main__":
    main()
